@@ -36,7 +36,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 pref("general.useragent.vendor", "Thunderbird");
-pref("general.useragent.vendorSub",
+pref("general.useragent.vendorSub", 
 #expand __APP_VERSION__
 );
 
@@ -50,7 +50,7 @@ pref("app.version",
 pref("app.build_id", 
 #expand __BUILD_ID__
 );
-pref("app.extensions.version", "0.8");
+pref("app.extensions.version", "1.0");
 
 #ifdef XP_MACOSX
 pref("mail.biff.animate_dock_icon", false);
@@ -113,7 +113,7 @@ pref("update.showSlidingNotification", true);   // Windows-only slide-up taskbar
 // 2 = high   (new version of Firefox/Security patch)
 pref("update.severity", 0); 
 
-pref("xpinstall.whitelist.add", "update.mozilla.org");
+pref("xpinstall.whitelist.add", "update.mozilla.org,addons.mozilla.org");
 
 /////////////////////////////////////////////////////////////////
 // Overrides of the seamonkey suite mailnews.js prefs
@@ -145,6 +145,7 @@ pref("addressbook.throbber.url","chrome://messenger-region/locale/region.propert
 
 // show a broken image icon for blocked remote image requests
 pref("network.image.imageBehavior", 2);
+pref("network.cookie.cookieBehavior", 2); // 0-Accept, 1-dontAcceptForeign, 2-dontUse, 3-p3p
 
 // thunderbird shows size in KB instead of lines by default
 pref("news.show_size_in_lines", false);
@@ -203,18 +204,23 @@ pref("offline.send.unsent_messages",            0);
 pref("offline.download.download_messages",  0);
 pref("offline.prompt_synch_on_exit",            true);
 
-// Expose only select protocol handlers. All others should go                   
-// through the external protocol handler route.                                 
-pref("network.protocol-handler.expose-all", false);                             
+// Expose only select protocol handlers. All others should go
+// through the external protocol handler route.
+pref("network.protocol-handler.expose-all", false);
 pref("network.protocol-handler.expose.mailto", true);
 pref("network.protocol-handler.expose.news", true);
 pref("network.protocol-handler.expose.snews", true);
 pref("network.protocol-handler.expose.nntp", true);
 pref("network.protocol-handler.expose.imap", true);
 pref("network.protocol-handler.expose.addbook", true);
-pref("network.protocol-handler.expose.pop", true);                                                                                                            
-pref("network.protocol-handler.expose.mailbox", true);  
-pref("network.protocols.useSystemDefaults",   false);  
+pref("network.protocol-handler.expose.pop", true);
+pref("network.protocol-handler.expose.mailbox", true);
+
+// suppress external-load warning for standard browser schemes
+pref("network.protocol-handler.warn-external.http", false);
+pref("network.protocol-handler.warn-external.https", false);
+pref("network.protocol-handler.warn-external.ftp", false);
+
 pref("network.hosts.smtp_server",           "mail");
 pref("network.hosts.pop_server",            "mail");
 
@@ -262,6 +268,8 @@ pref("browser.chrome.toolbar_style",        2);
 
 pref("browser.turbo.enabled", false);
 
+pref("browser.xul.error_pages.enabled", true);  
+
 // Dialog modality issues
 pref("browser.prefWindowModal", true);
 pref("browser.show_about_as_stupid_modal_window", false);
