@@ -186,10 +186,14 @@ function setServer(uri)
    if (msgFolder)
      msgFolder.getFilterList(gFilterListMsgWindow);
 
+   // this will get the deferred to account root folder, if server is deferred
+   msgFolder = msgFolder.server.rootMsgFolder;
+   var rootFolderUri = msgFolder.URI;
+
    rebuildFilterTree(uri);
    
    // root the folder picker to this server
-   gRunFiltersFolderPicker.setAttribute("ref", uri);
+   gRunFiltersFolderPicker.setAttribute("ref", rootFolderUri);
  
    // run filters after the fact not supported by news
    if (CanRunFiltersAfterTheFact(msgFolder.server)) {
@@ -502,7 +506,7 @@ function getSelectedServerForFilters()
         var msgFolder = selectedFolder.QueryInterface(Components.interfaces.nsIMsgFolder);
         try
         {
-            var rootFolder = msgFolder.rootFolder;
+            var rootFolder = msgFolder.server.rootFolder;
             if (rootFolder.isServer)
             {
                 var server = rootFolder.server;
