@@ -303,19 +303,15 @@ nsresult nsPop3IncomingServer::GetInbox(nsIMsgWindow *msgWindow, nsIMsgFolder **
   nsCOMPtr<nsIMsgLocalMailFolder> localInbox = do_QueryInterface(*inbox, &rv);
   if (NS_SUCCEEDED(rv) && localInbox)
   {
-    PRBool valid = PR_FALSE;
     nsCOMPtr <nsIMsgDatabase> db;
     rv = (*inbox)->GetMsgDatabase(msgWindow, getter_AddRefs(db));
-    if (NS_SUCCEEDED(rv) && db)
+    if (NS_FAILED(rv))
     {
-      rv = db->GetSummaryValid(&valid);
-      if (!valid)
-      {
-        (*inbox)->SetMsgDatabase(nsnull);
-        (void) localInbox->SetCheckForNewMessagesAfterParsing(PR_TRUE);
-        (*inbox)->GetMsgDatabase(msgWindow, getter_AddRefs(db));
-        return NS_MSG_ERROR_FOLDER_SUMMARY_OUT_OF_DATE;
-      }
+      (*inbox)->SetMsgDatabase(nsnull);
+      (void) localInbox->SetCheckForNewMessagesAfterParsing(PR_TRUE);
+      // this will cause a reparse of the mail folder.
+      localInbox->GetDatabaseWithReparse(nsnull, msgWindow, getter_AddRefs(db));
+      return NS_MSG_ERROR_FOLDER_SUMMARY_OUT_OF_DATE;
     }
   }
   return rv;
@@ -340,7 +336,11 @@ NS_IMETHODIMP nsPop3IncomingServer::PerformBiff(nsIMsgWindow *aMsgWindow)
     if (NS_FAILED(rv) || numFolders != 1) return rv;
   }
 
-  SetPerformingBiff(PR_TRUE);
+  nsCOMPtr <nsIMsgIncomingServer> server;
+  inbox->GetServer(getter_AddRefs(server));
+
+  server->SetPerformingBiff(PR_TRUE);
+
   urlListener = do_QueryInterface(inbox);
 
   PRBool downloadOnBiff = PR_FALSE;
@@ -626,10 +626,24 @@ nsPop3GetMailChainer::~nsPop3GetMailChainer()
 nsresult nsPop3GetMailChainer::GetNewMailForServers(nsISupportsArray *servers, nsIMsgWindow *msgWindow,
                                 nsIMsgFolder *folderToDownloadTo, nsIUrlListener *listener)
 {
+  NS_ENSURE_ARG_POINTER(folderToDownloadTo);
+
   m_serversToGetNewMailFor = servers;
   m_folderToDownloadTo = folderToDownloadTo;
   m_downloadingMsgWindow = msgWindow;
   m_listener = listener;
+  nsCOMPtr <nsIMsgDatabase> destFolderDB;
+
+  nsresult rv = folderToDownloadTo->GetMsgDatabase(msgWindow, getter_AddRefs(destFolderDB));
+  if (NS_FAILED(rv) || !destFolderDB)
+  {
+    nsCOMPtr <nsIMsgLocalMailFolder> localFolder = do_QueryInterface(folderToDownloadTo);
+    if (localFolder)
+    {
+      localFolder->GetDatabaseWithReparse(this, msgWindow, getter_AddRefs(destFolderDB));
+      return NS_OK;
+    }
+  }
   return RunNextGetNewMail();
 }
 
