@@ -474,6 +474,12 @@ function Startup()
 
   // initialize observers and listeners
   window.XULBrowserWindow = new nsBrowserStatusHandler();
+  window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+        .getInterface(Components.interfaces.nsIWebNavigation)
+        .QueryInterface(Components.interfaces.nsIDocShellTreeItem).treeOwner
+        .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+        .getInterface(Components.interfaces.nsIXULWindow)
+        .XULBrowserWindow = window.XULBrowserWindow;
 
   addPrefListener(gButtonPrefListener); 
   addPrefListener(gTabStripPrefListener);
@@ -737,6 +743,12 @@ function Shutdown()
 
   window.XULBrowserWindow.destroy();
   window.XULBrowserWindow = null;
+  window.QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+        .getInterface(Components.interfaces.nsIWebNavigation)
+        .QueryInterface(Components.interfaces.nsIDocShellTreeItem).treeOwner
+        .QueryInterface(Components.interfaces.nsIInterfaceRequestor)
+        .getInterface(Components.interfaces.nsIXULWindow)
+        .XULBrowserWindow = null;
 
   BrowserFlushBookmarksAndHistory();
 
@@ -2406,3 +2418,29 @@ function updateFileUploadItem()
   else
     item.setAttribute('disabled', 'true');
 }
+
+// Do a security check for drag n' drop. Make sure the source document
+// can load the dragged link.
+function DragDropSecurityCheck(aEvent, aDragSession, aUrl)
+{
+  var sourceDoc = aDragSession.sourceDocument;
+
+  if (sourceDoc) {
+    var sourceURI = sourceDoc.documentURI;
+
+    const nsIScriptSecurityManager =
+      Components.interfaces.nsIScriptSecurityManager;
+    var secMan = Components.classes["@mozilla.org/scriptsecuritymanager;1"]
+      .getService(nsIScriptSecurityManager);
+
+    try {
+      secMan.checkLoadURIStr(sourceURI, aUrl,
+                             nsIScriptSecurityManager.STANDARD);
+    } catch (e) {
+      // Stop even propagation right here.
+      aEvent.stopPropagation();
+
+      throw "Drop of " + aUrl + " denied.";
+    }
+  }
+}
